local OneChoice = LibStub('AceAddon-3.0'):NewAddon('OneChoice', 'AceEvent-3.0', 'AceHook-3.0')

local preferable_subclass_by_class, _
local CLOTH, LEATHER, MAIL, PLATE

function OneChoice:OnInitialize()
    -- This lets me get around having to ship a localization
    CLOTH   = self:GetSubclass("Cloth",   1433, 3642)
    LEATHER = self:GetSubclass("Leather", 1425, 2371) 
    MAIL    = self:GetSubclass("Mail", 	  2648, 2417) 
    PLATE   = self:GetSubclass("Plate",   8086, 8094)
    
    preferable_subclass_by_class = {
    	["HUNTER"] = LEATHER,
    	["WARLOCK"] = CLOTH,
    	["PRIEST"] = CLOTH,
    	["PALADIN"] = MAIL,
    	["MAGE"] = CLOTH,
    	["ROGUE"] = LEATHER,
    	["DRUID"] = LEATHER,
    	["SHAMAN"] = LEATHER,
    	["WARRIOR"] = MAIL,
    	["DEATHKNIGHT"] = PLATE,
    }
end

function OneChoice:OnEnable()
	self:SecureHook("QuestInfo_Display")
	self:RegisterEvent("PLAYER_LEVEL_UP") 
    self:PLAYER_LEVEL_UP(nil, UnitLevel("player"))
end

function OneChoice:GetSubclass(default, ...)
    for n = 1, 3 do
        for i = 1, select('#', ...) do 
            local subclass = select(7, GetItemInfo(select(i, ...)))
            if subclass then
                return subclass
            end         
        end
    end
	
	return default
end

function OneChoice:PLAYER_LEVEL_UP(event, level)
    if tonumber(level) > 39 then
        preferable_subclass_by_class["HUNTER"] = MAIL
        preferable_subclass_by_class["PALADIN"] = PLATE
        preferable_subclass_by_class["SHAMAN"] = MAIL
        preferable_subclass_by_class["WARRIOR"] = PLATE
    end
    
    local _, class = UnitClass("player")
    self.preferable_subclass = preferable_subclass_by_class[class]
end

function OneChoice:QuestInfo_Display()
    local number_of_choices
    
    if QuestInfoFrame.questLog then
        number_of_choices = GetNumQuestLogChoices() 
    else 
        number_of_choices = GetNumQuestChoices()
    end
    
    if number_of_choices < 2 then
        return
    end
    
    local most_valuable_choice, most_valuable_price = 0, 0

    for i = 1, number_of_choices do
        local link, numItems, isUsable
        
        if QuestInfoFrame.questLog then
            link = GetQuestLogItemLink("choice", i)
            _, _, numItems, _, isUsable = GetQuestLogChoiceInfo(i)
        else
            link = GetQuestItemLink("choice", i)
            _, _, numItems, _, isUsable = GetQuestItemInfo("choice", i)
        end
        
        if link then
            local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, texture, vendorPrice = GetItemInfo(link)
        
            if vendorPrice * numItems > most_valuable_price then
                most_valuable_choice = i
                most_valuable_price = vendorPrice
            end

            local universal_equip_slot = equipSlot == "INVTYPE_CLOAK" or equipSlot == "INVTYPE_FINGER" or equipSlot == "INVTYPE_NECK" or equipSlot == "INVTYPE_RELIC"
        
            if not universal_equip_slot and class == "Armor" and subclass ~= self.preferable_subclass then
                _G["QuestInfoItem"..i.."IconTexture"]:SetVertexColor(0.9, 0, 0)
    			_G["QuestInfoItem"..i.."NameFrame"]:SetVertexColor(0.9, 0, 0)
            end
        end

        _G["QuestInfoItem"..i.."Name"]:SetTextColor(1, 1, 1, 1)
    end
    
    local most_valued_texture = _G["QuestInfoItem"..most_valuable_choice.."IconTexture"]
    local most_valued_name = _G["QuestInfoItem"..most_valuable_choice.."Name"]
    
    if most_valued_texture and most_valued_name then
        most_valued_texture:SetVertexColor(1, 1, .26)
        most_valued_name:SetTextColor(1, 1, .26, 1)
    end
end
